package de.ozgcloud.eingang.enterprise.entry;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.router.VorgangService;
import lombok.SneakyThrows;

@ITCase
@AutoConfigureMockMvc
@ActiveProfiles({ "itcase", "local" })
class EnterpriseEntryITCase {

	@MockBean
	private VorgangService vorgangService;

	@Autowired
	private MockMvc mockMvc;

	@Test
	void shouldCallVorgangService() {
		doPostRequest();

		verify(vorgangService).createVorgang(any());
	}

	@SneakyThrows
	private ResultActions doPostRequest() {
		return mockMvc.perform(multipart("/antrag")
				.file(new MockMultipartFile("formData", "simple.json", MediaType.APPLICATION_JSON.toString(),
						TestUtils.loadTextFile("request/simple.json").getBytes())))
				.andExpect(status().is2xxSuccessful());
	}

	@Nested
	class A12Entry {

		@Captor
		private ArgumentCaptor<FormData> formDataCaptor;

		@Test
		void shouldCreateVorgang() {
			var formData = doPostRequest();

			assertThat(formData).isNotNull();
		}

		@Test
		void shouldHaveNoOfRepresentations() {
			var formData = doPostRequest();

			assertThat(formData.getNumberOfRepresentations()).isEqualTo(2);
		}

		@Test
		void shouldHaveFormData() {
			var formData = doPostRequest();

			assertThat(formData.getFormData()).containsKeys("Haushaltsjahr", "Anlage_1", "Anlage_2");
		}

		@Test
		void shouldHaveMukServiceKonto() {
			var formData = doPostRequest();

			assertThat(formData.getHeader().getServiceKonto().getType()).isEqualTo("MUK");
		}

		@SneakyThrows
		private FormData doPostRequest() {
			mockMvc.perform(multipart("/antrag")
					.file(buildJsonFileMock("formData", "a12_entry.json"))
					.file(buildJsonFileMock("representation", "document_JaS.json"))
					.file(buildJsonFileMock("representation", "jugendsozialarbeit_an_schulen-DM.json")))
					.andExpect(status().is2xxSuccessful());

			verify(vorgangService).createVorgang(formDataCaptor.capture());
			return formDataCaptor.getValue();
		}

	}

	private MockMultipartFile buildJsonFileMock(String name, String origName) {
		var fileToLoad = "request/%s".formatted(origName);
		return new MockMultipartFile(name, origName, MediaType.APPLICATION_JSON.toString(), TestUtils.loadTextFile(fileToLoad).getBytes());
	}
}
