package de.ozgcloud.eingang.fim;

import java.util.Map;
import java.util.Optional;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import lombok.extern.log4j.Log4j2;

@Log4j2
public abstract class FimSchemeAdapter {

	public abstract FimSchemeIdentifier forIdentifier();

	public Optional<String> getFieldName(final Map<String, Element> fieldIndex, final String fieldName) {
		final String[] fieldNameParts = fieldName.split(":");
		final String fieldNameWithoutNamespace = fieldNameParts[fieldNameParts.length - 1];
		if (!fieldIndex.containsKey(fieldNameWithoutNamespace)) {
			LOG.error("Cannot find Field: " + fieldName);
			return Optional.empty();
		}
		final Element nodeNameElement = fieldIndex.get(fieldNameWithoutNamespace);
		final Optional<String> nodeNameOpt = getNameForElement(nodeNameElement);
		return nodeNameOpt.map(this::cleanNodeName);
	}

	public Optional<String> getNameForElement(final Element element) {
		final NodeList nameTags = element.getElementsByTagName("name");
		if (nameTags.getLength() != 1) {
			return Optional.empty();
		}
		return Optional.ofNullable(nameTags.item(0).getTextContent());
	}

	public String cleanNodeName(final String s) {
		return s.trim();
	}
}
