package de.ozgcloud.eingang.fim;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import lombok.NonNull;

@Component
class ZustaendigeStelleMapper implements FimEngineBasedMapper {

	static final String XTA_IDENTIFIER_ENTRY_NAME = "xtaIdentifier";

	// TODO OZG-7086 umstellen auf FIM Headerdaten

	@Override
	public FormData parseFormData(FormData formData) {
		return formData.getControl().getMetaData()
				.map(metaData -> metaData.getEntry(XTA_IDENTIFIER_ENTRY_NAME))
				.filter(Objects::nonNull)
				.map(oeId -> setOrganisationsEinheitId(formData.getZustaendigeStelles(), oeId))
				.map(zustStelle -> formData.toBuilder().zustaendigeStelle(zustStelle).build())
				.orElse(formData);
	}

	private ZustaendigeStelle setOrganisationsEinheitId(List<ZustaendigeStelle> stelles, @NonNull String oeid) {
		var builder = stelles.isEmpty() ? ZustaendigeStelle.builder() : stelles.getFirst().toBuilder();

		extractOrganisationsEinheitId(oeid).ifPresent(builder::organisationseinheitenId);
		return builder.build();
	}

	Optional<String> extractOrganisationsEinheitId(@NonNull String xtaIdentifier) {
		var idx = xtaIdentifier.indexOf(":");
		if (idx < 0) {
			return Optional.empty();
		}
		return Optional.of(xtaIdentifier.substring(idx + 1));
	}
}
