package de.ozgcloud.eingang.formcycle.common.errorhandling;

import static de.ozgcloud.eingang.formcycle.common.errorhandling.InternalExceptionDtoTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.TechnicalException;
import io.grpc.StatusRuntimeException;

class FormcycleExceptionHandlerTest {

	@Spy
	@InjectMocks
	private FormcycleExceptionHandler exceptionHandler;

	@Nested
	class TestHandleTechnicalException {

		@Mock
		private TechnicalException technicalException;

		@Test
		void shouldCallBuildResponseEntity() {
			when(technicalException.getExceptionId()).thenReturn(EXCEPTION_ID);

			exceptionHandler.handleTechnicalException(technicalException, null);

			verify(exceptionHandler).buildResponseEntity(FormcycleExceptionHandler.TECHNICAL_EXCEPTION_MESSAGE, EXCEPTION_ID);
		}

	}

	@Nested
	class TestHandleStatusRuntimeException {

		@Mock
		private StatusRuntimeException statusRuntimeException;

		@Test
		void shouldCallBuildResponseEntity() {
			when(statusRuntimeException.getStackTrace()).thenReturn(new StackTraceElement[0]);
			when(statusRuntimeException.getMessage()).thenReturn(ExceptionUtil.formatMessageWithExceptionId(MESSAGE, EXCEPTION_ID));

			exceptionHandler.handleStatusRuntimeException(statusRuntimeException, null);

			verify(exceptionHandler).buildResponseEntity(FormcycleExceptionHandler.CREATE_VORGANG_EXCEPTION_MESSAGE, EXCEPTION_ID);
		}

		@Nested
		class TestGetExceptionId {

			@Test
			void shouldReturnExceptionIdFromMessage() {
				var exceptionId = exceptionHandler.getExceptionId(messageWithExceptionId());

				assertThat(exceptionId).isEqualTo(EXCEPTION_ID);
			}

			@Test
			void shouldCreateNewExceptionId() {
				var exceptionId = exceptionHandler.getExceptionId(MESSAGE);

				assertThat(exceptionId).isNotEqualTo(EXCEPTION_ID);
			}

		}

	}

	@Nested
	class TestBuildResponseEntity {

		@Test
		void shouldReturnInternaServerError() {
			var response = buildResponseEntity();

			assertThat(response.getStatusCode()).isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
		}

		@Test
		void shouldCallBuildInternalExceptionDto() {
			buildResponseEntity();

			verify(exceptionHandler).buildInternalExceptionDto(InternalExceptionDtoTestFactory.MESSAGE, EXCEPTION_ID);
		}

		private ResponseEntity<InternalExceptionDto> buildResponseEntity() {
			return exceptionHandler.buildResponseEntity(InternalExceptionDtoTestFactory.MESSAGE, EXCEPTION_ID);
		}

	}

	@Nested
	class TestBuildInternalExceptionDto {

		@Test
		void shouldSetExceptionId() {
			var response = buildInternalExceptionDto();

			assertThat(response.getExceptionId()).isEqualTo(EXCEPTION_ID);
		}

		@Test
		void shouldSetMessage() {
			var response = buildInternalExceptionDto();

			assertThat(response.getMessage()).isEqualTo(InternalExceptionDtoTestFactory.MESSAGE);
		}

		private InternalExceptionDto buildInternalExceptionDto() {
			return exceptionHandler.buildInternalExceptionDto(InternalExceptionDtoTestFactory.MESSAGE, EXCEPTION_ID);
		}

	}
}