package de.ozgcloud.eingang.common.zufi;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.eingang.common.zufi.OrganisationsEinheit;
import de.ozgcloud.eingang.common.zufi.OrganisationsEinheitMapper;
import de.ozgcloud.eingang.common.zufi.OrganisationsEinheitRemoteService;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;

class OrganisationsEinheitRemoteServiceTest {

	@InjectMocks
	private OrganisationsEinheitRemoteService service;
	@Mock
	private OrganisationsEinheitServiceBlockingStub stub;
	@Mock
	private OrganisationsEinheitMapper mapper;

	@DisplayName("Get by id")
	@Nested
	class TestGetById {

		private final GrpcOrganisationsEinheitGetResponse response = GrpcOrganisationsEinheitGetResponseTestFactory.create();
		private final OrganisationsEinheit mappedOrganisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void mock() {
			when(stub.getById(any())).thenReturn(response);
			when(mapper.fromGrpc(any())).thenReturn(mappedOrganisationsEinheit);
		}

		@Captor
		private ArgumentCaptor<GrpcOrganisationsEinheitGetRequest> requestCaptor;

		@Test
		void shouldCallRemoteService() {
			service.getById(OrganisationsEinheitTestFactory.ID);

			verify(stub).getById(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getId()).isEqualTo(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			service.getById(OrganisationsEinheitTestFactory.ID);

			verify(mapper).fromGrpc(response.getOrganisationsEinheit());
		}

		@Test
		void shouldReturnValue() {
			var organisationsEinheit = service.getById(OrganisationsEinheitTestFactory.ID);

			assertThat(organisationsEinheit).isEqualTo(mappedOrganisationsEinheit);
		}
	}
}
